%define anolis_release 4

Name:          unbound
Version:       1.17.1
Release:       %{anolis_release}%{?dist}
Summary:       Unbound is a validating, recursive, caching DNS resolver
License:       BSD
Url:           https://nlnetlabs.nl/projects/unbound/about/
Source:        https://nlnetlabs.nl/downloads/unbound/%{name}-%{version}.tar.gz
Source1: unbound.service
Source2: unbound.conf
Source3: unbound.munin
Source4: unbound_munin_
Source5: root.key
Source7: unbound-keygen.service
Source8: tmpfiles-unbound.conf
Source9: example.com.key
Source10: example.com.conf
Source11: block-example.com.conf
Source12: https://data.iana.org/root-anchors/icannbundle.pem
Source13: root.anchor
Source14: unbound.sysconfig
Source15: unbound-anchor.timer
Source16: unbound-munin.README
Source17: unbound-anchor.service
Source20: unbound.sysusers

Patch1000: backport-pre-CVE-2024-33655-extended_error_encode-for-extended-errors.patch 
Patch1001: backport-pre-CVE-2024-33655-Downstream-DNS-Cookies-a-la-RFC7873-and-RFC9018.patch
Patch1002: backport-pre-CVE-2024-33655-Fix-possibly-unaligned-memory-access-in-parse_edns_options_from_query.patch
Patch1003: backport-pre-CVE-2024-33655-Fix-out-of-bounds-read-in-parse_edns_options_from_query.patch
Patch1004: backport-CVE-2024-33655.patch

#https://github.com/NLnetLabs/unbound/commit/b7c61d7cc256d6a174e6179622c7fa968272c259
Patch1005: unbound-cve-2024-8508.patch

BuildRequires: gcc, make, systemd, pkgconfig(libsystemd), byacc 
BuildRequires: flex, swig, openssl-devel
BuildRequires: libevent-devel expat-devel
BuildRequires: python3-devel

Requires: %{name}-libs = %{version}-%{release}
Requires: %{name}-anchor = %{version}-%{release}
Recommends: %{name}-utils = %{version}-%{release}
# unbound-keygen.service requires it, bug #2116790
Requires: openssl

Obsoletes: unbound-help < 1.17.1
Conflicts: unbound-help < 1.17.1

%description
Unbound is a validating, recursive, caching DNS resolver. It is designed
to be fast and lean and incorporates modern features based on open standards.
To help increase online privacy, Unbound supports DNS-over-TLS which allows
clients to encrypt their communication. Unbound is available for most platforms
such as FreeBSD, OpenBSD, NetBSD, MacOS, Linux and Microsoft Windows.
Unbound is a totally free, open source software under the BSD license. It doesn't
make custom builds or provide specific features to paying customers only.

%package libs
Summary:       Libraries for %{name}
Recommends: %{name}-anchor
Requires(pre): systemd-sysusers

%description libs
Libraries for %{name}.

%package devel
Summary: Libraries and header files
Requires: %{name}-libs = %{version}-%{release}
Conflicts: unbound-help < 1.17.1

%description devel
Package devel includes libraries and header files for development with unbound.

%package anchor
Requires(pre): shadow-utils
Requires: %{name}-libs = %{version}-%{release}
Summary: DNSSEC trust anchor maintaining tool
Conflicts: unbound-help < 1.17.1
Conflicts: unbound-libs < 1.17.1
 
%description anchor
Contains tool maintaining trust anchor using RFC 5011 key rollover algorithm.
 
%package utils
Requires: %{name}-libs = %{version}-%{release}
Summary: Unbound DNS lookup utilities
Conflicts: unbound-help < 1.17.1
Conflicts: unbound < 1.17.1
 
%description utils
Contains tools for making DNS queries. Can make queries to DNS servers
also over TLS connection or validate DNSSEC signatures. Similar to
bind-utils.

%package -n python3-%{name}
Summary:       The python3 module of unbound DNS resolver
Requires:      %{name}-libs = %{version}-%{release}

%description -n python3-unbound
The python3 module of unbound DNS resolver.

%prep
%autosetup -p1 -n %{name}-%{version}

%build
%configure \
        --with-pythonmodule --with-pyunbound PYTHON=%{__python3} \
        --enable-systemd \
        --with-libevent --with-pthreads --with-ssl \
        --disable-rpath --disable-static \
        --enable-relro-now --enable-pie \
        --enable-subnet --enable-ipsecmod \
        --with-conf-file=%{_sysconfdir}/%{name}/unbound.conf \
        --with-pidfile=%{_rundir}/%{name}/%{name}.pid \
        --enable-sha2 --disable-gost --enable-ecdsa \
        --with-rootkey-file=%{_sharedstatedir}/unbound/root.key \
        --enable-linux-ip-local-port-range

%make_build
%make_build streamtcp

%install
install -p -m 0644 %{SOURCE16} .

%make_install unbound-event-install
install -m 0755 streamtcp $RPM_BUILD_ROOT%{_sbindir}/unbound-streamtcp

install -d -m 0755 %{buildroot}%{_unitdir} %{buildroot}%{_sysconfdir}/sysconfig
install -p -m 0644 %{SOURCE1} %{buildroot}%{_unitdir}/unbound.service
install -p -m 0644 %{SOURCE7} %{buildroot}%{_unitdir}/unbound-keygen.service
install -p -m 0644 %{SOURCE15} %{buildroot}%{_unitdir}/unbound-anchor.timer
install -p -m 0644 %{SOURCE17} %{buildroot}%{_unitdir}/unbound-anchor.service
install -p -m 0755 %{SOURCE2} %{buildroot}%{_sysconfdir}/unbound
install -p -m 0644 %{SOURCE12} %{buildroot}%{_sysconfdir}/unbound
install -p -m 0644 %{SOURCE14} %{buildroot}%{_sysconfdir}/sysconfig/unbound
install -p -D -m 0644 %{SOURCE20} %{buildroot}%{_sysusersdir}/%{name}.sysusers

# install streamtcp man page
install -m 0644 testcode/streamtcp.1 %{buildroot}/%{_mandir}/man1/unbound-streamtcp.1
install -D -m 0644 contrib/libunbound.pc %{buildroot}/%{_libdir}/pkgconfig/libunbound.pc

# Install tmpfiles.d config
install -d -m 0755 %{buildroot}%{_tmpfilesdir} %{buildroot}%{_sharedstatedir}/unbound
install -m 0644 %{SOURCE8} %{buildroot}%{_tmpfilesdir}/unbound.conf
 
# install root - we keep a copy of the root key in old location,
# in case user has changed the configuration and we wouldn't update it there
install -m 0644 %{SOURCE5} %{buildroot}%{_sysconfdir}/unbound/
install -m 0644 %{SOURCE13} %{buildroot}%{_sysconfdir}/unbound/dnssec-root.key
# make initial key static
pushd %{buildroot}%{_sharedstatedir}/unbound
  KEYPATH=$(realpath --relative-to="%{buildroot}%{_sharedstatedir}/unbound" "%{buildroot}%{_sysconfdir}/unbound/dnssec-root.key")
  ln -s "$KEYPATH" root.key
popd

mkdir -p %{buildroot}%{_rundir}/unbound
 
# Install directories for easier config file drop in
 
mkdir -p %{buildroot}%{_sysconfdir}/unbound/{keys.d,conf.d,local.d}
install -p %{SOURCE9} %{buildroot}%{_sysconfdir}/unbound/keys.d/
install -p %{SOURCE10} %{buildroot}%{_sysconfdir}/unbound/conf.d/
install -p %{SOURCE11} %{buildroot}%{_sysconfdir}/unbound/local.d/
 
# Link unbound-control-setup.8 manpage to unbound-control.8
echo ".so man8/unbound-control.8" > %{buildroot}/%{_mandir}/man8/unbound-control-setup.8

%pre libs
%sysusers_create_compat %{SOURCE20}
 
%post
%systemd_post unbound.service
%systemd_post unbound-keygen.service
 
%post anchor
%systemd_post unbound-anchor.service unbound-anchor.timer
# start the timer only if installing the package to prevent starting it, if it was stopped on purpose
if [ "$1" -eq 1 ]; then
    # the Unit is in presets, but would be started after reboot
    /bin/systemctl start unbound-anchor.timer >/dev/null 2>&1 || :
fi
 
%preun
%systemd_preun unbound.service
%systemd_preun unbound-keygen.service
 
%preun anchor
%systemd_preun unbound-anchor.service unbound-anchor.timer
 
%postun
%systemd_postun_with_restart unbound.service
%systemd_postun unbound-keygen.service
 
%postun anchor
%systemd_postun_with_restart unbound-anchor.service unbound-anchor.timer

%check
%make_build check

%files
%doc doc/CREDITS doc/FEATURES
%{_unitdir}/%{name}.service
%{_unitdir}/%{name}-keygen.service
%attr(0755,unbound,unbound) %dir %{_rundir}/%{name}
%attr(0644,root,root) %{_tmpfilesdir}/unbound.conf
%attr(0644,root,root) %config(noreplace) %{_sysconfdir}/%{name}/unbound.conf
%dir %attr(0755,root,unbound) %{_sysconfdir}/%{name}/keys.d
%attr(0644,root,unbound) %config(noreplace) %{_sysconfdir}/%{name}/keys.d/*.key
%dir %attr(0755,root,unbound) %{_sysconfdir}/%{name}/conf.d
%attr(0644,root,unbound) %config(noreplace) %{_sysconfdir}/%{name}/conf.d/*.conf
%dir %attr(0755,root,unbound) %{_sysconfdir}/%{name}/local.d
%attr(0644,root,unbound) %config(noreplace) %{_sysconfdir}/%{name}/local.d/*.conf
%ghost %attr(0640,root,unbound) %{_sysconfdir}/%{name}/unbound_control.pem
%ghost %attr(0640,root,unbound) %{_sysconfdir}/%{name}/unbound_control.key
%ghost %attr(0640,root,unbound) %{_sysconfdir}/%{name}/unbound_server.pem
%ghost %attr(0640,root,unbound) %{_sysconfdir}/%{name}/unbound_server.key
%{_sbindir}/unbound
%{_sbindir}/unbound-checkconf
%{_sbindir}/unbound-control
%{_sbindir}/unbound-control-setup
%{_mandir}/man5/*
%exclude %{_mandir}/man8/unbound-anchor*
%{_mandir}/man8/*

%files libs
%doc doc/README
%license doc/LICENSE
%attr(0755,root,root) %dir %{_sysconfdir}/%{name}
%{_sysusersdir}/%{name}.sysusers
%{_libdir}/libunbound.so.8*
%dir %attr(0755,unbound,unbound) %{_sharedstatedir}/%{name}
%verify(not size mtime filedigest link mode user group) %{_sharedstatedir}/%{name}/root.key
# just left for backwards compat with user changed unbound.conf files - format is different!
%attr(0644,root,root) %config %{_sysconfdir}/%{name}/root.key
%attr(0644,root,root) %config %{_sysconfdir}/%{name}/dnssec-root.key

%files -n python3-unbound
%license pythonmod/LICENSE
%{python3_sitearch}/*
%doc libunbound/python/examples/*
%doc pythonmod/examples/*

%files devel
%{_libdir}/libunbound.so
%{_includedir}/unbound.h
%{_includedir}/unbound-event.h
%{_mandir}/man3/*
%{_libdir}/pkgconfig/*.pc

%files anchor
%attr(0644,root,root) %config(noreplace) %{_sysconfdir}/sysconfig/%{name}
%{_sbindir}/unbound-anchor
%{_mandir}/man8/unbound-anchor*
# icannbundle and root.key(s) should be replaced from package
# intentionally not using noreplace
%config %{_sysconfdir}/%{name}/icannbundle.pem
%{_unitdir}/unbound-anchor.timer
%{_unitdir}/unbound-anchor.service
 
%files utils
%{_sbindir}/unbound-host
%{_sbindir}/unbound-streamtcp
%{_mandir}/man1/unbound-*

%changelog
* Wed Jan 15 2025 yangxinyu <yangxinyu@nfschina.com> - 1.17.1-4
- add patch to fix CVE-2024-8508

* Wed Jan 08 2024 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.17.1-3
- to #IBGLMU
- add patch to fix CVE-2024-33655
- Project: TC2024080204
- Signed-off-by: mgb01105731 mgb01105731@alibaba-inc.com

* Mon Mar 11 2024 Zhao Hang <wb-zh951434@alibaba-inc.com> - 1.17.1-2
- Rebuild with python3.11

* Wed Feb 15 2023 Funda Wang <fundawang@yeah.net> - 1.17.1-1
- New version 1.17.1

* Tue Mar 08 2022 Jacob Wang <yungao.jacob@linux.alibaba.com> - 1.15.0-1
- Initial version from upstream 1.15.0
